<?php

if (!defined('BOT_TOKEN')) {
   require_once __DIR__ . '/config.php';
}

function sendTelegramMessage($chatId, $message, $parseMode = 'Markdown') {
    $apiUrl = "https://api.telegram.org/bot" . BOT_TOKEN . "/sendMessage";
    $data = [
        'chat_id' => $chatId,
        'text' => $message,
        'parse_mode' => $parseMode
    ];
    $options = [
        'http' => [
            'method'  => 'POST',
            'header'  => "Content-Type: application/x-www-form-urlencoded\r\n",
            'content' => http_build_query($data),
            'ignore_errors' => true
        ]
    ];
    $context = stream_context_create($options);
    $resultRaw = @file_get_contents($apiUrl, false, $context);
    if ($resultRaw === false) {
        error_log("Gagal menghubungi API Telegram.");
        return false;
    }
    $result = json_decode($resultRaw, true);
    if (!$result || !isset($result['ok'])) {
        error_log("Gagal mengirim pesan Telegram. Response: " . $resultRaw);
        return false;
    }
    return $result['ok'];
}

function readJsonFile($filename) {
    if (!file_exists($filename)) return [];
    $jsonContent = @file_get_contents($filename);
    if ($jsonContent === false) return [];
    $data = json_decode($jsonContent, true);
    return (json_last_error() === JSON_ERROR_NONE && is_array($data)) ? $data : [];
}

function writeJsonFile($filename, $data) {
    $jsonData = json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    if ($jsonData === false) {
        error_log("Gagal encode JSON untuk file: " . $filename);
        return false;
    }
    if (@file_put_contents($filename, $jsonData, LOCK_EX) === false) {
        error_log("Gagal menulis ke file: " . $filename);
        return false;
    }
    return true;
}

function checkDomainStatus($domain) {
    $apiUrl = SKIDDLE_CHECK_URL . urlencode($domain);
    $responseRaw = @file_get_contents($apiUrl);
    if ($responseRaw === false) {
        return ['status' => 'error', 'message' => 'Gagal menghubungi API pengecekan.'];
    }
    $response = json_decode($responseRaw, true);
    if (json_last_error() !== JSON_ERROR_NONE || !is_array($response) || empty($response)) {
        return ['status' => 'error', 'message' => 'Format respons tidak valid dari API pengecekan.'];
    }
    $responseKey = array_key_first($response);
    if (isset($response[$responseKey]['blocked']) && $response[$responseKey]['blocked'] === true) {
        return ['status' => 'blocked', 'message' => '⚠️ Terdeteksi diblokir oleh KOMINFO'];
    } elseif (isset($response[$responseKey]['blocked'])) {
         return ['status' => 'clear', 'message' => '✅ Tidak terdeteksi diblokir'];
    } else {
        return ['status' => 'unknown', 'message' => 'Status tidak diketahui dari API.'];
    }
}

function addCloudflareCname($name, $content = CLOUDFLARE_DEFAULT_TARGET, $proxied = true) {
    if (!defined('CLOUDFLARE_API_TOKEN') || !defined('CLOUDFLARE_ZONE_ID') || empty(CLOUDFLARE_API_TOKEN) || empty(CLOUDFLARE_ZONE_ID)) {
         error_log("Konfigurasi Cloudflare tidak lengkap.");
         return null;
    }

    $apiUrl = "https://api.cloudflare.com/client/v4/zones/" . CLOUDFLARE_ZONE_ID . "/dns_records";
    $data = json_encode([
        'type' => 'CNAME',
        'name' => $name,
        'content' => $content,
        'ttl' => 1,
        'proxied' => $proxied
    ]);

    $options = [
        'http' => [
            'method' => 'POST',
            'header' => "Content-Type: application/json\r\n" .
                        "Authorization: Bearer " . CLOUDFLARE_API_TOKEN . "\r\n",
            'content' => $data,
            'ignore_errors' => true
        ]
    ];
    $context = stream_context_create($options);
    $resultRaw = @file_get_contents($apiUrl, false, $context);
    $result = json_decode($resultRaw, true);

    if ($result && isset($result['success']) && $result['success'] === true && isset($result['result']['id'])) {
        return $result['result']['id'];
    } else {
        error_log("Cloudflare API Error (Add CNAME for $name): " . $resultRaw);
        return null;
    }
}

function deleteCloudflareDnsRecord($dnsRecordId) {
     if (empty($dnsRecordId) || !defined('CLOUDFLARE_API_TOKEN') || !defined('CLOUDFLARE_ZONE_ID') || empty(CLOUDFLARE_API_TOKEN) || empty(CLOUDFLARE_ZONE_ID)) {
         error_log("Konfigurasi Cloudflare tidak lengkap atau ID DNS kosong untuk dihapus.");
         return false;
     }

    $apiUrl = "https://api.cloudflare.com/client/v4/zones/" . CLOUDFLARE_ZONE_ID . "/dns_records/" . $dnsRecordId;

    $options = [
        'http' => [
            'method' => 'DELETE',
            'header' => "Authorization: Bearer " . CLOUDFLARE_API_TOKEN . "\r\n",
            'ignore_errors' => true
        ]
    ];
    $context = stream_context_create($options);
    $resultRaw = @file_get_contents($apiUrl, false, $context);
    $result = json_decode($resultRaw, true);

    if ($result && isset($result['success']) && $result['success'] === true) {
        return true;
    } else {
        error_log("Cloudflare API Error (Delete DNS ID $dnsRecordId): " . $resultRaw);
        return false;
    }
}

function getWebhookInfo() {
    $apiUrl = "https://api.telegram.org/bot" . BOT_TOKEN . "/getWebhookInfo";
    $responseRaw = @file_get_contents($apiUrl);
    if ($responseRaw === false) {
        return ['ok' => false, 'error' => 'Gagal menghubungi API Telegram.'];
    }
    $response = json_decode($responseRaw, true);
    if (!$response || !isset($response['ok'])) {
        return ['ok' => false, 'error' => 'Respons tidak valid dari getWebhookInfo.', 'raw' => $responseRaw];
    }
    return $response;
}

function testCloudflareConnection() {
     if (!defined('CLOUDFLARE_API_TOKEN') || !defined('CLOUDFLARE_ZONE_ID') || empty(CLOUDFLARE_API_TOKEN) || empty(CLOUDFLARE_ZONE_ID)) {
         return ['ok' => false, 'error' => 'Konfigurasi Cloudflare (Token/Zone ID) tidak lengkap.'];
     }

    $apiUrl = "https://api.cloudflare.com/client/v4/zones/" . CLOUDFLARE_ZONE_ID . "/dns_records?per_page=1";

    $options = [
        'http' => [
            'method' => 'GET',
            'header' => "Authorization: Bearer " . CLOUDFLARE_API_TOKEN . "\r\n" .
                        "Content-Type: application/json\r\n",
            'ignore_errors' => true
        ]
    ];
    $context = stream_context_create($options);
    $resultRaw = @file_get_contents($apiUrl, false, $context);
    $result = json_decode($resultRaw, true);

    if ($result && isset($result['success']) && $result['success'] === true) {
        return ['ok' => true, 'message' => 'Koneksi API Cloudflare Berhasil (Zona: ' . CLOUDFLARE_ZONE_ID . ')'];
    } elseif ($result && isset($result['errors']) && !empty($result['errors'])) {
        $errorMsg = $result['errors'][0]['message'] ?? 'Error tidak diketahui';
        $errorCode = $result['errors'][0]['code'] ?? 'N/A';
         return ['ok' => false, 'error' => "Cloudflare API Error (Code: $errorCode): $errorMsg"];
    } else {
        return ['ok' => false, 'error' => 'Gagal menghubungi atau respons tidak dikenal dari API Cloudflare.', 'raw' => $resultRaw];
    }
}

function isUserAuthorized($userId) {
    if (empty(AUTHORIZED_USER_IDS)) {
        return true;
    }
    return in_array($userId, AUTHORIZED_USER_IDS);
}

?>
